<?php

class GoogleFeedLib {

	public function generate() {
		ini_set('memory_limit', '5000M');
		set_time_limit(0);

		$siteTitle = Configure::read('SiteSetting.general.site_title');

		// build the url
		$siteUrl = Configure::read('App.fullBaseUrl');

		$xml = new XMLWriter;
		$xml->openMemory();
		$xml->startDocument( '1.0', 'UTF-8');

		$xml->startElement( 'rss' );
			$xml->writeAttribute( 'version', '2.0' );
			$xml->writeAttribute( 'xmlns:g', 'http://base.google.com/ns/1.0' );

			$xml->startElement( 'channel' );
				$xml->writeElement( 'title', $siteTitle);
				$xml->writeElement( 'link', urlencode($siteUrl) );
				$xml->writeElement( 'description', $siteTitle . ' Product Feed' );

				// get the products
				// Additional query parameters to fetch out any data we need for the feed.
				$params = array(
					'conditions' => array(
						'Product.is_active' => 1,
					),
					//'order' => 'CategoriesProduct.sequence ASC, Product.name ASC',
					'contain' => array(
						'ListingImage',
						'Variant' => array(
							'order' => 'Variant.sequence ASC',
							'VariantPricing'
						),
						'Brand',
					),
					'group' => 'Product.id',
					'fields' => array(
						'*'
					)
				);
				if (CakePlugin::loaded('EvInventory')) {
					$params['contain']['Variant'][] = 'Inventory';
				}

				$this->_modifyProductQueryParams($params);

				$ProductModel = EvClassRegistry::init('EvShop.Product');
				$VariantModel = EvClassRegistry::init('EvShop.Variant');
				$VariantPricingModel = EvClassRegistry::init('EvShop.VariantPricing');

				if (CakePlugin::loaded('EvInventory')) {
					$VariantModel->linkInventory();
				}

				$products = $ProductModel->find('all', $params);

				foreach ($products as $product) {
					$variantCount = count($product['Variant']);

					$productUrl = RouterUtil::getItemRoute('EvShop', 'Product');
					$productUrl[] = $product['Product']['id'];

					$url = Router::url($productUrl);

					$currencies = Configure::read('currencies');

					foreach ($product['Variant'] as $variant) {

						if (! isset($variant['VariantPricing'][0]['price']) || $variant['VariantPricing'][0]['price'] <= 0
							|| empty($variant['sku'])) {
							continue;
						}

						$variant = $VariantPricingModel->calculateTax($variant, $ProductModel);

						$xml->startElement('item');

							$xml->writeElement('g:id', $product['Product']['id'] . '_' . $variant['id']);

							$title = $product['Product']['name'];
							if ($variantCount > 1) {
								// if we have actual variants append the variant name
								$title .= ': ' . $variant['name'];
							}

							$title = preg_replace("/[^a-z0-9\s-_.,&+]/i", "", $title);
							$xml->writeElement('g:title', $title);

							$description = strip_tags($product['Product']['description']);
							$description = preg_replace('/[\x00-\x1F\x80-\xFF]/', '', $description);
							$xml->startElement('g:description');
								$xml->writeCData($description);
							$xml->endElement();

							// @todo: categories
							$xml->writeElement( 'g:google_product_category', Configure::read('EvShop.googleFeedCategory'));

							// check for a category
							if (isset($product['CategoriesProduct']['0']['Category']['name']) && ! empty($product['CategoriesProduct']['0']['Category']['name'])) {
								$xml->writeElement('g:product_type', $product['CategoriesProduct']['0']['Category']['name']);
							}

							$xml->writeElement('g:link', $siteUrl . $url);

							// check for an image
							if (
								isset($product['ListingImage']['0']['filepath']) &&
								! empty($product['ListingImage']['0']['filepath']) &&
								file_exists(WWW_ROOT . $product['ListingImage']['0']['filepath'])
							) {

								$image = ltrim($product['ListingImage']['0']['filepath'], '/');
								$xml->writeElement('g:image_link', $siteUrl . '/' . $image);
							}

							$xml->writeElement('g:condition', 'new');

							if (CakePlugin::loaded('EvInventory')) {
								if ($variant['Inventory']['stock'] > 0) {
									$stock = 'in stock';
								} else {
									$stock = 'out of stock';
								}
							} else {
								$stock = 'in stock';
							}

							$xml->writeElement('g:availability', $stock);
							$xml->writeElement('g:price', number_format($variant['VariantPricing'][0]['price_incTax'], 2) . ' ' . $currencies[$variant['VariantPricing'][0]['currency_id']]);
							$xml->writeElement('g:mpn', $variant['sku']);

							if ($variant['VariantPricing'][0]['sale_price_incTax'] > 0) {
								$xml->writeElement('g:sale_price', number_format($variant['VariantPricing'][0]['sale_price_incTax'], 2) . ' GBP');
							}

							if (isset($product['Brand']['name']) && ! empty($product['Brand']['name'])) {
								$xml->writeElement('g:brand', $product['Brand']['name']);
							}

							$this->_addAdditionalXmlProperties($xml, $variant, $product);

						$xml->endElement(); // </item>
					}
				}

			$xml->endElement(); // </channel>
		$xml->endElement(); // </rss>
		$xml->endDocument();

		$output = $xml->outputMemory();
		return $output;
	}

/**
 * Override this to modify the product query
 * @param  [type] &$params The current query params
 */
	protected function _modifyProductQueryParams(&$params) {
		// Does nothing by default
		return;
	}

/**
 *  Override to add additional properties into the generated xml file
 * @param [type] &$xml    The xml document that is being constructed
 * @param [type] $variant The current variant
 * @param [type] $product The product this variant belongs to
 */
	protected function _addAdditionalXmlProperties(&$xml, $variant, $product) {
		// Does nothing by default
		return;
	}

}
